## Simple models of continuous trait evolution: Brownian motion, Ornstein-Uhlenbeck and Early burst models.
## Written by Samantha Price - March 2015, for the Bodega Applied Phylogenetics Workshop

source('~/Desktop/Bodega2015/ContinuousTraits/Functions_ContTraitSim.R', chdir = TRUE)

# ==How to simulate Brownian motion==

# What is Brownian motion?  At each instant in time the state of a character can increase or decrease and the direction and magnitude of these changes are independent of current or past states (no trends etc.).  The total displacement of a character after time t is drawn from a normal distribtion with a mean of zero (i.e. net change in trait is 0) and a variance proportional to t. 

#1) Non-phylogenetic simulation.

# First generate the displacements of the trait using the rnorm function to generate a normal distribution with a mean of zero for 100 units of time.

displace<-rnorm(100) 
plot(displace)

## The sum of these displacements represents the trajectory of the trait through time

x<-cumsum(displace)
plot(x, type="l", xlab="Time", ylab="Trait Value")

## Now create a plot with 50 separate runs of Brownian motion. You can imagine these as 50 independent species (sharing no common ancestry).
cols<-rainbow(50) # put 50 samples from the rainbow pallette into a new vector called cols
plot(cumsum(rnorm(100)), type="l", ylim=c(-30,30))
for(i in 1:length(cols)) lines(cumsum(rnorm(100)), col=cols[i]) # simple for loop for each of the 50 colour samples we draw a line on the plot that is the cumulative summation of 100 draws from a normal distribution. 

# Here we can see variance in the trait is proportional to time 

# The standard deviation of the normal distribution is related to the Brownian motion rate parameter. Thus, if your rate is faster you generate more variance aka disparity. Examine this by running a simulation that first runs with sd=1 (the default) for 500 generations and then sd=5 for 100 generations and look at the variance

plot(cumsum(rnorm(100)), type="l", ylim=c(-80,80), ylab="Trait", col="blue")
for(i in 1:9) lines(cumsum(rnorm(500)), col="blue")
for(i in 1:10) lines(cumsum(rnorm(100, sd=5)), col="green")

#2) How to run a phylogenetic simulation

# Obviously the above simulations were assuming the species were independent but we know that they are not, they share evolutionary history and this affects the evolution of their traits. Think about a 3 species tree:

tree<-read.tree(text="((grizzlybear:1, polarbear:1):4, spectacledbear:5);")
plot(tree)
axisPhylo()
nodelabels()
# Looking at this tree we see that grizzly bears and polar bears share about 4 million years of history during which they all follow the same BM trajectory after which grizzly and polar bears split with both lineages starting on a new BM trajectory - we can simulate this. We need the branch length information: the longer the branch the greater the expected variance - with this we determine the standard deviation of the normal distribution. The total displacement of a character after time v is drawn from a normal distribtion with a mean of zero (i.e. net change in trait is 0) and a standard deviation = sqrt(rate*branchlength). 

# Lets assume we have a root state of 1 and a sigma^2 of 0.5 

# Start with the first branch from the root node 4 to 5 and estimate the trait value at node 5 and use the branch lengths to estimate the displacement.

x<-rnorm(1, mean=0, sd=sqrt(0.5*tree$edge.length[1]))

# You obviously need to convert this displacement to a trajectory, so if the root state is 1 then you add the displacement to the root state to get the value at the node.

node5est<-1+x

# You then repeat this process across the tree calculating the displacement with sqrt(0.5*tree$edge.length[i]) and adding it to the ancestral node. Fortunately you don't need to be able to do this as there are many R functions that will do it for you such as fastBM in phytools.

# ==How to simulate Brownian motion with a trend == 
# Look at the code for the bmtrend model in Functions_ContTraitSim.R. It simply uses a Brownian motion model but the mean increases or decreases over time as given by the trend parameter mu. 

# Run bmtrend function 10 times using the replicate function, once with a mu of 1 and the other with a mu=-1

bmtrend0.002<-replicate(10, bmtrend(100, 1, 1))
bmtrendneg0.002<-replicate(10, bmtrend(100, -1, 1))


plot(bmtrend0.002[,1], xlim=c(0,100), ylim=c(-150, 150), type="l", col="red", xlab="Time", ylab="Trait")
for(i in 2:10) lines(bmtrend0.002[,i], col="red")
for(i in 1:10) lines(bmtrendneg0.002[,i], col="blue")


# ==How to simulate Ornstein-Uhlenbeck ==

# Look at the code for the Ornstein-Uhlenbeck model in Functions_ContTraitSim.R. You can see it simply converts the equation into R code but also includes how many generations you want to run (time) and the starting value for the trait (x0). 

# Spend a five minutes exploring what happens when you change alpha, sigma, theta and the ancestral value using the ornstein_uhlenbecksim function



ouhialpha<-replicate(10, ornstein_uhlenbecksim(100, theta=2, alpha=0.5, sigma2=0.5, x0=1))
oulowalpha<-replicate(10, ornstein_uhlenbecksim(100, theta=2, alpha=0.05, sigma2=0.5, x0=1))


plot(ouhialpha[,1], xlim=c(0,100), ylim=c(-150, 150), type="l", col="red", xlab="Time", ylab="Trait")
for(i in 2:10) lines(ouhialpha[,i], col="red")
for(i in 1:10) lines(oulowalpha[,i], col="purple")
legend("topleft", c("alpha 0.5", "alpha=0.05"), col=c("red", "purple"))

# Reminder: the above simulations were assuming the species were independent but we know that they are not, they share evolutionary history and this affects the evolution of their traits. So when simulating along a phylogeny time is also an important component, to be able to account for this you need to include time and dt back into the equations. Fortunately, you don't need to be able to do this yourself as the R package OUwie has a simulation function OUwie.sim. With an ultrametric tree the maximum likelihood estimates of theta and the ancestral values are the same under a single optima model but if you have fossils that is no longer true. 

# ==How to simulate Early or Late Bursts ==

# Look at the code for the eblb function in Functions_ContTraitSim.R. Again this model is a modification of the simple Brownian motion model - the rate changes exponentially over time (t). So to simulate it you need the time, the initial starting rate and the exponential change parameter r, the standard deviation of the Brownian motion model is modified so that sd= initial rate * e(-rt) 

# Spend a few minutes exploring what happens when you change r and sigma. It is a little temperamental due to the difficulties of using small numbers for the sd in rnorm so you have to chose your starting sigma and r very carefully (-0.05 and 0.05 work....)

eblb1<-replicate(10, eblb(100, 0.05, sigma0=0.5))
eblb2<-replicate(10, eblb(100, -0.05, sigma0=0.5))


plot(eblb1[,1], xlim=c(0,100), ylim=c(-150, 150), type="l", col="cyan2", xlab="Time", ylab="Trait")
for(i in 2:10) lines(eblb1[,i], col="cyan2")
for(i in 1:10) lines(eblb2[,i], col="purple")
legend("topleft", c("r=0.05", "r=-0.05"), col=c("red", "purple"))