## B) Fitting models of continuous trait evolution and exploring patterns in data
## Written by Samantha Price - March 2015, for the Bodega Applied Phylogenetics Workshop

library(phytools)
library(geiger)
library(picante)

# Read in the cetacean tree and lifehistory dataset, 

cetphy<-read.nexus("~/Desktop/Bodega2015/ContinuousTraits/Datasets/CetaceaPhy.nex") # for the purposes of this tutorial I am going to assume that you have a folder called Bodega2015 on your desktop and in it you have stored the dataset folder and tutorial you downloaded from the website, if you have stored it elsewhere/under a different name you will need to change the path.

cetdat<-read.table("~/Desktop/Bodega2015/ContinuousTraits/Datasets/CetaceaLH.txt", header=T)

tmp<-treedata(cetphy, cetdat, sort=T) # useful function in geiger that creates an object with the overlapping taxa in the tree and the dataset

tr<-tmp[[1]]
dat<-tmp[[2]]

## INDEPENDENT CONTRASTS

## Independent contrasts divide up the phylogeny into "independent" contrasts so we can identify a relationship between the traits independent of phylogeny. If we have a phylogeny of 4 species: A & B are sister taxa as are C & D, then there are 3 independent comparisons, one between A & B, one between C & D and finally, one between A&B and C&D the contrasts are calculated as the difference between the trait values from these comparisons. Brownian motion is used to standardize the contrast, as sister-taxa that have had more time to evolve independently are expected to have larger contrasts, so the contrasts are divided by the sqrt of the of the sum of the branch lengths between (standard deviation of the contrast). Also contrasts deeper in the phylogeny are "corrected", as deeper within the phylogeny our methods of estimating the trait values becomes more uncertain as we are estimating the trait value thus we downweight the internal contrasts. This was explained on in the lecture in more detail.

## Using the pic function in ape calculate the contrasts for body size and gestation length and plot them against one another as well as run the regression on the contrasts. Because the sign of the contrasts is arbitrary the regression must go through the origin which you can do in R using lm(equation, -1) 

# create vectors for both data
logmass<-log10(dat[,1])
names(logmass)<-row.names(dat)
logsmlength<-log10(dat[,2])
names(logsmlength)<-row.names(dat)

## A correlation/regression etc. with PIC is necessary when the residuals from the OLS regression are phylogenetically patterned, such that the distribution is predicted by a Brownian motion model upon the phylogeny. This can be checked by estimating Blomberg's K (Blomberg, S. P., and T. Garland, Jr. 2002. Journal of Evolutionary Biology 15:899-910.), it is the ratio of the observed statistic to that expected under Brownian motion thus K= 1 suggests perfect Brownian motion, K < 1 weaker signal than expected under BM or stronger if K > 1 (i.e. more closely related species are even more similar in their traits than you expect from Brownian motion). The actual statistic is the mean squared error of the tip data divided by the phylogenetically corrected mean squared error of the data. This is implemented in the Kcalc function in the picante package. You can compare it to the expected value if you shuffled the tips on the phylogeny i.e. break up any signal using the phylosignal function in the picante package

residbmsml<-resid(lm(logsmlength~logmass))

phylosignal(residbmsml, tr)

# so although the strength of the covariance amongst species is not as great as expected under BM it is still significantly different from random, so we will complete our independent contrasts.

picbm<-pic(logmass, tr, scaled=T, var.contrasts=T) # we want them to be scaled by the branch length so scaled=T which is the default, we also want them to give us the standard deviation i.e. sqrt of the branch length with var.contrasts=T

picsml<-pic(logsmlength, tr, scaled=T, var.contrasts=T)

piclmbmsml<-lm(picsml[,1] ~ picbm[,1] -1 )# plot the contrasts forcing the line through the origin.
summary.lm(piclmbmsml) # look at the amount of variance explained now we have correctly included phylogeny it is significantly more than we saw when we did not include phylogeny in our calculations.
plot(picbm[,1], picsml[,1], xlab="Body mass PIC", ylab="Female length at sexual maturity PIC")
abline(piclmbmsml)

## Checking the contrasts have been appropriately standardized
#  If we have correctly standardized our contrasts under Brownian motion there should no longer be any relationship between the absolute value (i.e. ignoring the sign) of the standardized contrast and the age of the node. We can access this using the function branching.times in the ape package. This can be very informative about model of evolution, Brownian or not!

par(mfrow=c(1,2))

plot(branching.times(tr), abs(picsml[,1]), main="Female length sexual maturity")
cor.test(branching.times(tr), abs(picsml[,1]))

plot(branching.times(tr), abs(picbm[,1]), main="Adult body mass")
cor.test(branching.times(tr), abs(picbm[,1]))

# Why does Brownian motion specifically predict that the absolute value of the standardize contrast should not be related to age? So what does it mean if the absolute contrasts of Female length at sexual maturity are much higher closer to the root?  That more variance was generated early in the evolutionary history of the clade. Think about the Brownian motion model, variance is proportional to time & what else? The rate. The faster the rate the greater the variance. So what may have generated greater variance close to the root - faster rates! But a strict Brownian model means constant rates, so if there is a positive or negative relationship between the age of the node and the absolute contrast it suggests that the rate has changed over time. 

# What do you do if the contrasts show non-Brownian evolution? Well, remember if you are running a correlation/regression etc. it only matters if the residuals are Brownian. If the residuals are not Briw then you may need to use a different model. This can be done by rescaling the branch lengths to fit the pattern or you can implement a phylogenetic generalized least squares analyses specifying a non-Brownian variance covariance matrix as part of the error term. So how can we know the best model that fits the data?

## VISUALIZATION
# We have already seen that inspecting a plot of the absolute value of independent contrasts vs node age can be particularly informative when trying to infer potential models of evolution. Another very useful technique are traitgrams or phenograms as they are called in phytools. These project phylogenetic tree into the space defined by the phenotype on the y axis and time on the x-axis using ML ancestral state reconstructions .

phenogram(tr, logmass)

# Another useful technique is the phylomorphospace (), which projects the phylogenetic structure into the space defined by two phenotypes. 

phylomorphospace(tr, cbind(logmass, logsmlength))

#Although this particular example is not so helpful due to the incredibly strong correlation between the two traits. However generally,  if you find a region of the space occupied by very different lineages it suggests there may be phenotypic convergence, while if different lineages occupy different regions of the space it suggests phenotypic divergence.


## MODEL FITTING
# Also perhaps the most obvious way to know which model fits your data best is to actually estimate the model-fit! We can compare the fit of various models using the fitContinuous function in geiger. 

#Using the adult body mass data fit a Brownian motion model, a single-peak OU model and an Early/Late Burst model as well as a non-phylogenetic white noise model which assumes a single normal distribution of the data with no phylogenetic covariance amongst species and determine which fits the data best. When using the Akaike Information Criterion (corrected for small sample size AICc) the rule of thumb is a difference between the best fitting model (the lowest AICc score) which is referred to as the deltaAIc of 2 or more suggest some support for one model over another and >10 is substantial support. 

mass_bm<-fitContinuous(tr, logmass, model="BM")
mass_ou<-fitContinuous(tr, logmass, model="OU")
mass_eb<-fitContinuous(tr, logmass, model="EB", bounds=list(a=c(-5,5))) # the defaults fix it to only allow an early burst model with decelerating rates (max of the a rate parameter is -0.0000001). So if you want to also fit a late burst model with accelerating rates you have to change the bounds to allow positive numbers.  
mass_wn<-fitContinuous(tr, logmass, model="white")

mass_bm$opt$aicc
mass_ou$opt$aicc
mass_eb$opt$aicc
mass_wn$opt$aicc

# How can you interpret these results? Think about how the models are related to one another and look at the various parameter estimates by 
mass_bm$opt
mass_ou$opt
mass_eb$opt
mass_wn$opt

# As clearly noted in the help manual for fitContinuous you need to be aware that the "difficulty in finding the optimal solution is determined by an interaction between the nature and complexity of the likelihood space (which is data- and model-dependent) and the numerical optimizer used to explore the space. There is never a guarantee that the optimal solution is found, but using many random starting points (control$niter) and many optimization methods (control$method) will increase these odds."


##########EXCERCISE: IDENTIFY THE MODEL OF EVOLUTION###################

# Put all the techniques you have learnt to the test by exploring the evolutionary patterns within 5 traits and decide which model of evolution they evolved under. As these models are stochastic I have simulated 35 instances of each trait, each person will analyze a unique trait and we will see if the class comes to a consensus. Each trait has its own file: trait1.txt to trait5.txt. If you are very comfortable with R and you have plenty of time you could loop through all of the instances. 

# What would you conclude based just on model-fitting and do alter your conclusions if you try the various visualization techniques? 





