## Complex models of continuous trait evolution: multi-rate and multipeak models using the OUwie package.
## Written by Samantha Price - March 2015, for the Bodega Applied Phylogenetics Workshop

library(phytools)
library(OUwie)

# Read in the marsupial tree and dataset

mtr<-read.nexus("~/Desktop/Bodega2015/ContinuousTraits/Datasets/marsupialtr1.nex") # for the purposes of this tutorial I am going to assume that you have a folder called Bodega2015 on your desktop and in it you have stored the dataset folder and tutorial you downloaded from the website, if you have stored it elsewhere/under a different name you will need to change the path.

marsupiald<-read.table("~/Desktop/Bodega2015/ContinuousTraits/Datasets/marsupialmass_diet.txt", header=T)


# First check the data by plotting the distribution of diet on tips of the tree - we are going to quickly gloss over this as we covered this exact same example in the discrete trait tutorial a couple of days ago.

#Set up a vector of colours to represent the 3 diets.

colr<-rep("green", length(marsupiald[,1])) # set up a vector of "green" for the length of the tip labels this will act as the diet =1
colr[marsupiald[,2]==2]<-"purple" # where diet = 2 replace "green" with purple
colr[marsupiald[,2]==3]<-"blue" # where diet = 3 replace "green" with blue
names(colr)<-marsupiald[,1] # name the colour vector with the name of each species
colr<-colr[mtr$tip.label] # sort the colours so they match the same order as the tip labels in the tree

# plot the tree and replace the species names with coloured dots to represent diet
plot(mtr, show.tip.label=F)
tiplabels(pch=16, col=colr)

# Read in the tree we created in the discrete trait tutorial with the maximum likelihood state for diet at each node.

dietmtr<-read.nexus("~/Desktop/Bodega2015/DiscreteTraits/Datasets/marsupialdiettree.nex")

nodecolr<-rep("green", length=mtr$Nnode)
nodecolr[dietmtr$node.label==2]<-"purple"
nodecolr[dietmtr$node.label==3]<-"blue"

plot(dietmtr, show.tip.label=F)
tiplabels(pch=16, col=colr)
nodelabels(pch=16, col=nodecolr)

# Now lets run a single model in OUwie, we are going to run the single-peak OU model= OU1


sizedietou1<-OUwie(dietmtr, marsupiald, model=c("OU1"),root.station=TRUE, diagn=T)

# What are we doing when we set root.station=TRUE? We are stating whether the starting state, theta_0, should be estimated and the default is TRUE, which means theta_0 is dropped from the model so the starting value is distributed according to the stationary distribution of the OU process. Dropping theta_0 from the model can stabilize estimates of the primary optima, especially if you find the estimates of theta in the full model do not make sense biologically. However, this assumption does not fit a biological scenario involving moving away from an ancestral state. A good idea is to run preliminary analyses using both TRUE and FALSE to see which works for your dataset, for this one TRUE is best.

# What are we doing when we set diagn=TRUE? These models we are fitting are very complex and sometimes the information contained within the data are not sufficient and as a result one or more parameters may be estimated incorrectly. If you include the diagnostics in the OUwie run (diagn=T) it will include the eigendecomposition of the Hession matrix, which can indicate whether the maximum likelihood estimate has been found. If the eigenvalues of the Hessian are positive then parameter estimates are considered reliable (i.e. the Hessian is positive definite). 

# Look at the output

sizedietou1

# This tells you all the basic information about model fit (likelihood and AIC) and the parameter estimates. To extract individual elements you will need to look at the structure of the output

str(sizedietou1)

# You can see this is a much longer list of useful information, including the diagnostics from the Hessian, which are in the eigval element

sizedietou1$eigval

# Is there are better way of checking the results and the influence of the likelihood surface on the parameter estimates? PARAMETRIC BOOTSTRAPPING! OUwie now has a built in function to run this for you.... 

#When theta0 was dropped from the model (theta0=T) i.e. not estimated, then we need to set the root as the the value of the continuous trait for the selective regime mapped at the root, but as we are only using a single-peak model there is only one anyway! sizedietou1$theta[1,1]. This is a simulation approach so you want to run 100's-1000's but we don't have time so we are going to run 10!

sizedietou1boot10<-OUwie.boot(dietmtr, marsupiald, model=c("OU1"), nboot=10, alpha=sizedietou1$solution[1,] , sigma.sq=sizedietou1$solution[2,], theta=sizedietou1$theta[,1], theta0= sizedietou1$theta[1,1])

apply(sizedietou1boot10, 2, quantile, probs=c(0.025,0.975)) # calculate 95% CI from the bootstrap runs for each parameter - if the likelihood surface is flat you will get wide confidence intervals..

# Now run OUwie for every model - we can save some typing by writing a simple loop across all the models.

mods<-c("BM1", "OU1", "BMS", "OUM", "OUMA", "OUMV") # We set up a vector containing the names of the models we want to run and use this in a simple loop. You will notice this time we are not going to run the OUMVA as this model is generally very difficult to fit to this data and it may not run at all.

sizedietresults<-list()# setting up an empty list for the results
for(i in 1:length(mods)){
	sizedietresults[[i]]<-OUwie(dietmtr, marsupiald, model=mods[i], simmap.tree=FALSE, root.station=TRUE, diagn=T)
}

# First look to see if the Hessian is positive definite given in $eigval, the results are in a list so you need to extract the AICc from all 6 list elements (one for each analysis), to do this you use [[]]

hessian<-rbind(sizedietresults[[1]]$eigval, sizedietresults[[2]]$eigval, sizedietresults[[3]]$eigval, sizedietresults[[4]]$eigval, sizedietresults[[5]]$eigval, sizedietresults[[6]]$eigval) # as these are not all the same length as it gives an estimate for each parameter estimated it will give a warning that the number of columns of the result is not a multiple of vector length but this won't matter as we are just interested in making sure they are all positive. This is the quick way - use parametric bootstrapping for publications if at all possible..... (sometimes datasets are so large and the analyses so slow this isn't practical)
row.names(hessian)<-mods

# No look at the AICc, the results are in a list so you need to extract the AICc from all 6 list elements (one for each analysis), to do this you use [[]]
dietbestfit<-c(sizedietresults[[1]]$AICc, sizedietresults[[2]]$AICc, sizedietresults[[3]]$AICc, sizedietresults[[4]]$AICc, sizedietresults[[5]]$AICc, sizedietresults[[6]]$AICc)
names(dietbestfit)<-mods# results are in the same order in which we ran the models

dietbestfit-min(dietbestfit)

# All three models that fit separate optima for each diet category fit equally well, lets look at the estimated optima, to see which diet has the largest mass - 1, 2, or 3. Include the standard errors of the theta estimate also estimated from the Hessian matrix. If the SE are large it indicates that if you change the parameter value in any direction it will have little effect on the log-likelihood and the parameter estimate is considered unstable. 

bestfitthetas<-cbind(sizedietresults[[4]]$theta, sizedietresults[[5]]$theta, sizedietresults[[6]]$theta)
colnames(bestfitthetas)<-c("OUMtheta", "OUMthetaSE", "OUMAtheta", "OUMAthetaSE", "OUMVtheta", "OUMVthetaSE") # $theta contains both the estimated theta and the Standard Error so we are going to have 6 columns when we extract the result from the 3 multi-optima OU models. These SE look to be okay as well and even including the SE the rate estimates rarely overlap.

## These result are dependent on the tree topology, branch lengths and ancestral state reconstruction, which we know are not estimated without error. Instead we could use a Bayesian method such as stochastic character mapping to generate a set of dietary histories on a sample of phylogenies from the posterior distribution generated in BEAST or MrBayes. We can then run the analyses across this set of maps and summarize the results - although this may be a conservative estimate of the variability (Revell 2013).


########################################################
# How to run OUwie across a set of stochastically mapped                 #
# phylogenies.                                                                                       #
#                                                                                                            #
# This is provided as a reference, as depending                                  #
# on your R knowledge, it can sometimes be hard to take the             #
# simple exercises we give during the workshop and apply it to          #
# a set of trees etc..                                                                              #
# For example, it is very important to be careful when looping over    #
# trees as the structure of the output may change depending on the  #
# tree. For example, OUwie and Brownie.lite do not automatically     #
# sort their results to give the output for each state 1, 2, 3, instead it #
# is determined by the stochastic character maps at the root, which  #
# is variable. This is why it is always a good idea to sort the results   #
# using the order function.                                                                   #
# This code uses a nested loop, in R you could make this much        #
#  neater using the apply functions. However for teaching I prefer     #
# to use loops as they are universal coding oncepts and I believe     #
# concepts and I believe more intuitive and easier to read for            #
# beginners                                                                                         #
#######################################################



# Read in 100 stochastically mapped trees that we generated in our discrete trait tutorial. Obviously you would want to use many more trees than this in a real analysis but we don't have the time! We can tell OUwie we have stochastically mapped trees by using simmap.tree=T

marsupialtrs<-read.simmap("~/Desktop/Bodega2015/DiscreteTraits/Datasets/marsupialdietsimmap.tre", format="phylip", rev.order=T) #
# lets look at some of the trees to make sure everything looks as we would expect
par(mfrow=c(2,2))
plotSimmap(marsupialtrs[[1]], fsize=0)
plotSimmap(marsupialtrs[[3]], fsize=0)
plotSimmap(marsupialtrs[[6]], fsize=0)
plotSimmap(marsupialtrs[[8]], fsize=0)


# Now run all six models over the set of trees - I recommend taking a subset of 10 simmap trees to practice, as depending on your computer it will take between 20-60 mins to run on 10. We create an output table (resmat) of all the results we are interested in for each tree, run the set of six models and fill out the table, save the table to the results list (marsupialresults) and then move to the next tree. So the outer loop is the one that iterates over the trees and the inner loop goes over the models. Unfortunately, it is not quite as simple as that as we actually have to loop over the Brownian and OU models separately due to difficult thetas. 

marsupialtrs<-marsupialtrs[1:10]

## IMPORTANT REITERATION. It is very important to be careful when looping over trees as the structure of the output may change depending on the tree. For example, OUwie and Brownie.lite do not automatically sort their results to give the output for each state 1, 2, 3 instead it is determined by the stochastic character maps, which is variable. This is why we sort the results using the order function. 


mods<-c("BM1", "BMS","OU1", "OUM", "OUMA", "OUMV") # Don't forget there is the OUMVA model it just won't converge with this dataset..
marsupialresults<-list()

for(i in 1:length(marsupialtrs)){
	print(i)#helps to know how far through the analyses it is
	resmat<-matrix(nrow=21, ncol=6) # set up an empty results matrix for all the parameter estimates you are interested in, for each tree a new matrix will be created
	colnames(resmat)<-c("BM1", "BMS", "OU1", "OUM", "OUMA", "OUMV")
	row.names(resmat)<-c("loglik", "AICc", "Herb_alph", "Herb_alphaSE","Herb_sigma", "Herb_sigmaSE", "Omn_alph", "Omn_alphaSE","Omn_sigma", "Omn_sigmaSE", "Carn_alph", "Carn_alphaSE","Carn_sigma", "Carn_sigmaSE", "Herb_theta", "Herb_thetaSE", "Omn_theta", "Omn_thetaSE", "Carn_theta", "Carn_thetaSE", "MLStatus")

		for(j in 3:6){ # running OU models first
	
		tmpres<-OUwie(marsupialtrs[[i]], marsupiald, model=mods[j], simmap.tree=TRUE, root.station=TRUE, diagn=T) # we want the diagnostics on, as you want to be able to evaluate the ML estimate. We also want to indicate that this time we are using stochastically mapped trees with simmap.tree=T
	
		ev<-vector() # 
			for(k in 1:length(tmpres$eigval)) if (tmpres$eigval[k]>0) ev<-c(ev,0) else ev<-c(ev,1) # this is a looped if statement, it evaluates simple logical statements to see if any of the eigenvalues are negative, if not a 0 is added to the vector ev (if (tmpres$eigval[k]>0) ev<-c(ev,0))but if it is negative a 1 is added (else ev<-c(ev,1)).
	
		eval<-vector()
		if(sum(ev)==0) eval<-"All eigenvalues positive" else eval<-"Some eigenvalues negative" # if there are any eigenvalues that are negative the sum of the ev vector will be greater than 1.
	
	#We need to sort the results as mapped.edge determines the order, this may be problematic when we are summarizing across different maps as the the results are not always given in the order 1,2,3 but the sorting fixes this issue.

		row.names(tmpres$theta)<-colnames(tmpres$solution)
		tmpres$theta<-tmpres$theta[order(row.names(tmpres$theta)),]
		tmpres$solution<-tmpres$solution[,order(colnames(tmpres$solution))] 
		tmpres$solution.se<-tmpres$solution.se[,order(colnames(tmpres$solution.se))]
	
		resmat[,j]<-c(tmpres$loglik, tmpres$AICc, tmpres$solution[1], tmpres$solution.se[1], tmpres$solution[2], tmpres$solution.se[2], tmpres$solution[3], tmpres$solution.se[3], tmpres$solution[4], tmpres$solution.se[4], tmpres$solution[5], tmpres$solution.se[5], tmpres$solution[6], tmpres$solution.se[6], tmpres$theta[1,], tmpres$theta[2,], tmpres$theta[3,], eval)
	
	}
	
	for(l in 1:2){ # running BM models second - we don't want any thetas in our results, OUwie gives output in the theta column which are the phylogenetic means but as there is not pull towards the mean they cannot really be considered 'optima'
		tmpres<-OUwie(marsupialtrs[[i]], marsupiald, model=mods[l], simmap.tree=TRUE, root.station=TRUE, diagn=T) # we want the diagnostics on, as you want to be able to evaluate the ML estimate. We also want to indicate that this time we are using stochastically mapped trees with simmap.tree=T
	
		ev<-vector() # 
			for(k in 1:length(tmpres$eigval)) if (tmpres$eigval[k]>0) ev<-c(ev,0) else ev<-c(ev,1) # this goes through and evaluates whether any of the eigenvalues are negative, if is not a 0 is added to the vector ev but if it is negative a 1 is added 
	
		eval<-vector()
		if(sum(ev)==0) eval<-"All eigenvalues positive" else eval<-"Some eigenvalues negative" # if there are any eigenvalues that are negative the sum of the ev vector will be greater than 1.
	
	#We need to sort the results as mapped.edge determines the order, this may be problematic when we are summarizing across different maps as the the results are not always given in the order 1,2,3 but the sorting fixes this issue.

		tmpres$solution<-tmpres$solution[,order(colnames(tmpres$solution))] 
		tmpres$solution.se<-tmpres$solution.se[,order(colnames(tmpres$solution.se))]
	
		resmat[,l]<-c(tmpres$loglik, tmpres$AICc, tmpres$solution[1], tmpres$solution.se[1], tmpres$solution[2], tmpres$solution.se[2], tmpres$solution[3], tmpres$solution.se[3], tmpres$solution[4], tmpres$solution.se[4], tmpres$solution[5], tmpres$solution.se[5], tmpres$solution[6], tmpres$solution.se[6],0,0,0,0,0,0, eval)# 0's are for the thetas
		
	}
	
marsupialresults[[i]]<-resmat
}

save(marsupialresults, file="marsupialresults.Rdata")

# First make sure all the ML estimates are okay according to the eigenvalues - you want to remove the results of any that say "Some eigenvalues negative" - we shouldn't have any of these.

errors<-matrix(nrow=length(marsupialtrs), ncol=ncol(marsupialresults[[1]]))
colnames(errors)<-colnames(marsupialresults[[1]])
for(i in 1:length(marsupialtrs)) errors[i,]<-marsupialresults[[1]][21,]


# Now calculate the delta AICc for the different models across all trees
AICctable<-matrix(nrow=length(marsupialtrs), ncol=ncol(marsupialresults[[1]]))
colnames(AICctable)<-colnames(marsupialresults[[1]])
for(i in 1:length(marsupialtrs)) AICctable[i,]<-as.numeric(marsupialresults[[i]][2,])-as.numeric(min(marsupialresults[[i]][2,]))
	
colMeans(AICctable)  # you can see the best-fitting model is not the one found with the maximum likelihood reconstructions at the node

# As the best-fitting model is OUM (which fits different optima to each dietary category but with a single rate and alpha) lets look at the theta estimates for the OUM. Perhaps a better option might be to calculate the model-averaged thetas, whereby you sum the thetas across all models weighted by the Akaike Weight for each model.  

herbivoreoptima<-vector()
omnivoreoptima<-vector()
carnivoreoptima<-vector()

for(i in 1:length(marsupialresults)){
	herbivoreoptima<-c(herbivoreoptima,as.numeric(marsupialresults[[i]][15,4]))
	omnivoreoptima<-c(omnivoreoptima,as.numeric(marsupialresults[[i]][17,4]))
	carnivoreoptima<-c(carnivoreoptima,as.numeric(marsupialresults[[i]][19,4]))
}


hist(herbivoreoptima, xlim=c(0,12), ylim=c(0, 8), col="green", main="Optimal Mass from Best-fitting model", xlab="Body mass optima (ln mass (g))")
hist(carnivoreoptima, xlim=c(0,12), col="blue", add=T)
hist(omnivoreoptima, xlim=c(0,12), col="purple", add=T, breaks=3)
